@@ -12,13 +12,13 @@
 
 (macsyma-module grind)
 
-(declare-top (special lop rop *grind-charlist* chrps $aliases aliaslist linel))
+(declare-top (special lop rop *grind-charlist* chrps $aliases linel))
 
 (defun chrct* () (- linel chrps))
 
 (defmspec $grind (x)
   (setq x (cdr x))
-  (let (y)
+  (let (($lispdisp t) y)
     (fresh-line)
     (cond
       ((null x))
@@ -130,10 +130,10 @@
        (do () ((null x) l) (setq l (string1 (car x) l) x (cdr x))))))
 
 (defun msize (x l r lop rop)
-  (setq x (nformat x))
+  (setq x (nformat-check x))
   (cond ((atom x) (msize-atom x l r))
         ((and (atom (car x)) (setq x (cons '(mprogn) x)) nil))
-	((or (<= (lbp (caar x)) (rbp lop)) (> (lbp rop) (rbp (caar x))))
+	((or (<= (lbp (caar x)) (rbp lop)) (>= (lbp rop) (rbp (caar x))))
 	 (msize-paren x l r))
 	((member 'array (cdar x) :test #'eq) (msize-array x l r))
 	((safe-get (caar x) 'grind)
@@ -154,14 +154,12 @@
            ((and (setq y (safe-get x 'reversealias))
                  (not (and (member x $aliases :test #'eq) (get x 'noun))))
             (setq y (exploden (stripdollar y))))
-           ((setq y (rassoc x aliaslist :test #'eq))
-            (return (msize (car y) l r lop rop)))
            ((null (setq y (exploden x))))
            ((safe-get x 'noun) (return (msize-atom (get x 'noun) l r)))
            ((char= #\$ (car y)) (setq y (slash (cdr y))))
            ((member (marray-type x) '(array hash-table $functional))
             (return (msize-array-object x l r)))
-           (t (setq y (cons #\? (slash y)))))
+           (t (setq y (if $lispdisp (cons #\? (slash y)) (slash y)))))
      (return (msz y l r))))
 
 (defun msz (x l r)
@@ -188,8 +186,8 @@
 ;; communication.  The FORTRAN program rebinds them to #/( and #/) since
 ;; Fortran array references are printed with parens instead of brackets.
 
-(defvar lb #\[)
-(defvar rb #\])
+(defvar *lb* #\[)
+(defvar *rb* #\])
 
 (defun msize-array (x l r &aux f)
   (if (eq (caar x) 'mqapply) (setq f (cadr x) x (cdr x)) (setq f (caar x)))
@@ -200,8 +198,8 @@
 	      (not (member (caar x) (cdr $aliases) :test #'eq))
 	      (not (get (caar x) 'reversealias)))
 	 (setq l (cons #\' l))))
-  (setq l (msize f l (list lb) lop 'mfunction)
-	r (msize-list (cdr x) nil (cons rb r)))
+  (setq l (msize f l (list *lb*) lop 'mfunction)
+	r (msize-list (cdr x) nil (cons *rb* r)))
   (cons (+ (car l) (car r)) (cons l (cdr r))))
 
 (defun msize-function (x l r op)
@@ -275,11 +273,11 @@
 
 ;;; ----------------------------------------------------------------------------
 
-;; Formating a mlable-expression
+;; Formating a mlabel-expression
 
-(defprop mlable msize-mlable grind)
+(defprop mlabel msize-mlabel grind)
 
-(defun msize-mlable (x l r)
+(defun msize-mlabel (x l r)
   (declare (special *display-labels-p*))
   (if *display-labels-p*
       (setq l (cons (msize (cadr x) (list #\( ) (list #\) #\ ) nil nil) l)))
@@ -297,12 +295,12 @@
       (msz nil l r)
       (do ((nl) (w 0))
           ((null (cdr x))
-           (setq nl (cons (if (atom (car x))
+           (setq nl (cons (if (stringp (car x))
                               (msz (makestring (car x)) l r)
                               (msize (car x) l r lop rop))
                           nl))
            (cons (+ w (caar nl)) (nreverse nl)))
-        (setq nl (cons (if (atom (car x))
+        (setq nl (cons (if (stringp (car x))
                            (msz (makestring (car x)) l r)
                            (msize (car x) l r lop rop))
                        nl)
@@ -388,7 +386,7 @@
 
 (defun msz-mexpt (x l r)
   (setq l (msize (cadr x) l nil lop 'mexpt)
-	r (if (mmminusp (setq x (nformat (caddr x))))
+	r (if (mmminusp (setq x (nformat-check (caddr x))))
 	      (msize (cadr x) (reverse '(#\^ #\-)) r 'mexpt rop)
 	      (msize x (list #\^) r 'mexpt rop)))
   (list (+ (car l) (car r)) l r))
