@@ -133,7 +133,7 @@
 			   (car all)))))))
 
 (defun $backtrace (&optional (n 30))
-  (let ($display2d (st *debug-io*))
+  (let ($display2d)
     (loop for i below n
 	   for j from *current-frame*
 	   while (print-one-frame j t))))
@@ -405,10 +405,10 @@
       (catch *quit-tag*
 	(unwind-protect
 	     (do () (())
-	       (format *debug-io*
-		       "~a~&~@[(~a:~a) ~]~a"  *prompt-prefix* 
-		       (unless (stringp at) "dbm")
-		       (length *quit-tags*) *prompt-suffix*)
+	       (format-prompt *debug-io* "~a"
+			      (format nil "~&~@[(~a:~a) ~]"
+				      (unless (stringp at) "dbm")
+				      (length *quit-tags*)))
 	       (finish-output *debug-io*)
 	       (setq val
 		     (catch 'macsyma-quit
@@ -419,6 +419,8 @@
 							 (cdr res) 
 				                         'break-command)))
 				  (cond ((eq value :resume) (return)))))
+			       ((eq res *top-eof*)
+			        (funcall (get :top 'break-command)))
 			       (t
 				(setq $__ (nth 2 res))
 				(setq $% (meval* $__))
@@ -648,7 +650,8 @@ Command      Description~%~
 
 (defmacro lisp-quiet (&rest l)
   (setq *mread-prompt* "")
-  (eval (cons 'progn l)))
+  (eval (cons 'progn l))
+  nil)
 
 (def-break :lisp-quiet 'lisp-quiet 
   "Evaluate the lisp form without printing a prompt")
